@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Daniele E. Domenichelli <daniele.domenichelli@gmail.com>
+ * Copyright (C) 2011, 2012 Daniele E. Domenichelli <daniele.domenichelli@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,17 +25,28 @@
 #include <KWebView>
 #include <KWebPage>
 
-#include <QtWebKit/QWebSettings>
 #include <QtGui/QProgressBar>
 #include <QtGui/QBoxLayout>
 #include <QtGui/QLayout>
 #include <QtNetwork/QNetworkReply>
 
+#include <QtWebKit/QWebSettings>
+#include <QtWebKit/QWebFrame>
+
+#include <qjson/parser.h>
+
 
 const QLatin1String msnClientID("000000004C070A47");
+const QLatin1String scopes("wl.messenger wl.offline_access");
 const QLatin1String redirectUri("https://oauth.live.com/desktop");
-const QLatin1String request("https://oauth.live.com/authorize?response_type=token&redirect_uri=%1&client_id=%2&scope=wl.messenger");
-const QLatin1String tokenParameter("access_token=");
+const QLatin1String tokenUri("https://oauth.live.com/token");
+
+const QLatin1String authorizeRequest("https://oauth.live.com/authorize?client_id=%1&scope=%2&response_type=code&redirect_uri=%3");
+const QLatin1String tokenRequest("https://oauth.live.com/token?client_id=%1&redirect_uri=%2&code=%3&grant_type=authorization_code");
+
+const QLatin1String codeParameter("code");
+const QLatin1String accessTokenParameter("access_token");
+const QLatin1String refreshTokenParameter("refresh_token");
 
 XMessengerOAuth2Prompt::XMessengerOAuth2Prompt(QWidget* parent) :
     KDialog(parent),
@@ -97,7 +108,7 @@ XMessengerOAuth2Prompt::XMessengerOAuth2Prompt(QWidget* parent) :
             SLOT(onUnsupportedContent(QNetworkReply*)));
 
     // start loading the login URL
-    KUrl url(QString(request).arg(redirectUri).arg(msnClientID));
+    KUrl url(QString(authorizeRequest).arg(msnClientID).arg(scopes).arg(redirectUri));
     m_webView->load(url.url());
 }
 
@@ -110,6 +121,11 @@ QByteArray XMessengerOAuth2Prompt::accessToken() const
     return m_accessToken;
 }
 
+QByteArray XMessengerOAuth2Prompt::refreshToken() const
+{
+    return m_refreshToken;
+}
+
 QSize XMessengerOAuth2Prompt::sizeHint() const
 {
     return QSize(500, 600);
@@ -122,7 +138,8 @@ void XMessengerOAuth2Prompt::onUrlChanged(const QUrl &url)
         // This is not the url containing the token
         return;
     }
-    extractToken(url);
+    kDebug() << "This is the url we are waiting for";
+    extractCode(url);
 }
 
 void XMessengerOAuth2Prompt::onLinkClicked(const QUrl& url)
@@ -137,6 +154,12 @@ void XMessengerOAuth2Prompt::onLoadFinished(bool ok)
         m_loginPageLoaded = true;
         Q_EMIT loginPageLoaded(ok);
     }
+    if (m_webView->url().toString().indexOf(tokenUri) != 0) {
+        // This is not the url containing the token
+        return;
+    }
+    extractTokens(m_webView->page()->currentFrame()->toPlainText());
+
 }
 
 void XMessengerOAuth2Prompt::onUnsupportedContent(QNetworkReply* reply)
@@ -146,37 +169,66 @@ void XMessengerOAuth2Prompt::onUnsupportedContent(QNetworkReply* reply)
     // request
     // FIXME: In the future this might want to remove this
     QUrl url =  reply->url();
+    kDebug() << url;
     if (url.toString().indexOf(redirectUri) != 0) {
         // This is not the url containing the token
         return;
     }
-    extractToken(url);
+    extractCode(url);
 }
 
-void XMessengerOAuth2Prompt::extractToken(const QUrl &url)
+void XMessengerOAuth2Prompt::extractCode(const QUrl &url)
 {
-    QString accessToken;
-    Q_FOREACH (const QString &token, QString::fromLatin1(url.encodedFragment()).split(QLatin1Char('&'))) {
-        // Get the URL fragment part and iterate over the parameters of the request
-        if (token.indexOf(tokenParameter) == 0) {
-            // This is the token that we are looking for (we are not interested
-            // in the "expires_in" part, etc.)
-            accessToken = token;
-            break;
-        }
-    }
-    if (accessToken.isEmpty()) {
-        // Could not find the token
-        kWarning() << "Token not found";
+    kDebug() << url;
+    QString code = url.queryItemValue(codeParameter);
+
+    if (code.isEmpty()) {
+        // Could not find the access token
+        kWarning() << "Code not found";
         return;
     }
 
-    accessToken = accessToken.split(QLatin1Char('=')).at(1);    // Split by "access_token=..." and take latter part
+    kDebug() << "Code found:" << code;
+
+    // start loading the login URL
+    KUrl tokenUrl(QString(tokenRequest).arg(msnClientID).arg(redirectUri).arg(code));
+    m_webView->load(tokenUrl.url());
+}
+
+
+void XMessengerOAuth2Prompt::extractTokens(const QString &text)
+{
+    kDebug() << text;
+
+    QJson::Parser parser;
+    bool ok;
+
+    QVariantMap result = parser.parse(text.toLatin1(), &ok).toMap();
+    if (!ok) {
+        kWarning() << "An error occured during parsing reply";
+        return;
+    }
+
+    if (!result.contains(refreshTokenParameter)) {
+        // Could not find the refresh token
+        kWarning() << "Refresh token not found";
+        return;
+    }
+
+    m_refreshToken = result[refreshTokenParameter].toByteArray();
+    Q_EMIT refreshTokenTaken(m_refreshToken);
+
+    if (!result.contains(accessTokenParameter)) {
+        // Could not find the access token
+        kWarning() << "Access token not found";
+        return;
+    }
 
     // Wocky will base64 encode, but token actually already is base64, so we
     // decode now and it will be re-encoded.
-    m_accessToken = QByteArray::fromBase64(QByteArray::fromPercentEncoding(accessToken.toUtf8()));
+    m_accessToken = QByteArray::fromBase64(QByteArray::fromPercentEncoding(result[accessTokenParameter].toByteArray()));
     Q_EMIT accessTokenTaken(m_accessToken);
+
     accept();
 }
 
